@@ -19,9 +19,6 @@
 
 #ifdef USE_LIBLOG
 #define LOG_TAG "usbhost"
-#include "utils/Log.h"
-#define D ALOGD
-#else
 #define D printf
 #endif
 
