@@ -21,8 +21,10 @@
 #include <mtp.h>
 #include <MtpDatabase.h>
 #include <MtpDataPacket.h>
+#include <MtpStringBuffer.h>
 #include <MtpObjectInfo.h>
 #include <MtpProperty.h>
+#include <MtpDebug.h>
 
 #include <cstdlib>
 #include <cstring>
@@ -31,12 +33,22 @@
 #include <vector>
 #include <string>
 #include <tuple>
+#include <exception>
+#include <sys/inotify.h>
 
+#include <boost/thread.hpp>
+#include <boost/asio.hpp>
+#include <boost/bind.hpp>
+#include <boost/algorithm/string.hpp>
+#include <boost/assign/list_of.hpp>
 #include <boost/foreach.hpp>
 #include <boost/filesystem.hpp>
 #include <boost/range/adaptors.hpp>
 #include <boost/range/algorithm.hpp>
 
+#include <glog/logging.h>
+
+namespace asio = boost::asio;
 using namespace boost::filesystem;
 
 namespace android
@@ -46,17 +58,105 @@ private:
     struct DbEntry
     {
         MtpStorageID storage_id;
-        std::string object_name;
         MtpObjectFormat object_format;
         MtpObjectHandle parent;
         size_t object_size;
         std::string display_name;
         std::string path;
+        int watch_fd;
     };
 
+    MtpServer* local_server;
     uint32_t counter;
     std::map<MtpObjectHandle, DbEntry> db;
+    std::map<std::string, MtpObjectFormat> formats = boost::assign::map_list_of
+        (".gif", MTP_FORMAT_GIF)
+        (".png", MTP_FORMAT_PNG)
+        (".jpeg", MTP_FORMAT_JFIF)
+        (".tiff", MTP_FORMAT_TIFF)
+        (".ogg", MTP_FORMAT_OGG)
+        (".mp3", MTP_FORMAT_MP3)
+        (".wav", MTP_FORMAT_WAV)
+        (".wma", MTP_FORMAT_WMA)
+        (".aac", MTP_FORMAT_AAC)
+        (".flac", MTP_FORMAT_FLAC);
+
+    boost::thread notifier_thread;
+    boost::thread io_service_thread;
+
+    asio::io_service io_svc;
+    asio::posix::stream_descriptor stream_desc;
+    asio::streambuf buf;
+    int inotify_fd;
+
+    MtpObjectFormat guess_object_format(std::string extension)
+    {
+        std::map<std::string, MtpObjectFormat>::iterator it;
+
+        it = formats.find(extension);
+        if (it == formats.end()) {
+            boost::to_upper(extension);
+            it = formats.find(extension);
+            if (it == formats.end()) {
+                return MTP_FORMAT_UNDEFINED;
+            }
+	}
+
+	return it->second;
+    }
+
+    int setup_dir_inotify(path p)
+    {
+        return inotify_add_watch(inotify_fd,
+                                 p.string().c_str(),
+                                 IN_MODIFY | IN_CREATE | IN_DELETE);
+    }
+
     
+    void add_file_entry(path p, MtpObjectHandle parent)
+    {
+        MtpObjectHandle handle = counter;
+        DbEntry entry;
+
+        counter++;
+
+        if (is_directory(p)) {
+            entry.storage_id = MTP_STORAGE_FIXED_RAM;
+            entry.parent = parent;
+            entry.display_name = std::string(p.filename().string());
+            entry.path = p.string();
+            entry.object_format = MTP_FORMAT_ASSOCIATION;
+            entry.object_size = 0;
+            entry.watch_fd = setup_dir_inotify(p);
+
+            db.insert( std::pair<MtpObjectHandle, DbEntry>(handle, entry) );
+
+            if (local_server)
+                local_server->sendObjectAdded(handle);
+
+            parse_directory (p, handle);
+        } else {
+            try {
+                entry.storage_id = MTP_STORAGE_FIXED_RAM;
+                entry.parent = parent;
+                entry.display_name = std::string(p.filename().string());
+                entry.path = p.string();
+                entry.object_format = guess_object_format(p.extension().string());
+                entry.object_size = file_size(p);
+
+                VLOG(1) << "Adding \"" << p.string() << "\"";
+
+                db.insert( std::pair<MtpObjectHandle, DbEntry>(handle, entry) );
+
+                if (local_server)
+                    local_server->sendObjectAdded(handle);
+
+            } catch (const filesystem_error& ex) {
+                PLOG(WARNING) << "There was an error reading file properties";
+            }
+        }
+    }
+
     void parse_directory(path p, MtpObjectHandle parent)
     {
 	DbEntry entry;
@@ -66,31 +166,7 @@ private:
 
         for (std::vector<path>::const_iterator it(v.begin()), it_end(v.end()); it != it_end; ++it)
         {
-            MtpObjectHandle handle = counter;
-
-            counter++;
-
-            std::cout << "   " << *it << " handle: " << handle << " parent: " << parent << std::endl;
-
-            entry.storage_id = MTP_STORAGE_FIXED_RAM;
-            entry.parent = parent;
-            entry.object_name = it->filename().string();
-            entry.display_name = it->filename().string();
-            entry.path = it->string();
-
-            if (is_regular_file (*it)) {
-                entry.object_format = MTP_FORMAT_DEFINED;
-                entry.object_size = file_size(*it);
-
-                db.insert( std::pair<MtpObjectHandle, DbEntry>(handle, entry) );
-            } else if (is_directory (*it)) {
-                entry.object_format = MTP_FORMAT_ASSOCIATION;
-                entry.object_size = 0;
-
-                db.insert( std::pair<MtpObjectHandle, DbEntry>(handle, entry) );
-
-                parse_directory (*it, handle);
-	    }
+            add_file_entry(*it, parent);
         }
     }
 
@@ -103,46 +179,135 @@ private:
         try {
             if (exists(p)) {
                 if (is_directory(p)) {
-                    std::cout << p << " is a directory containing:\n";
-
                     entry.storage_id = MTP_STORAGE_FIXED_RAM;
                     entry.parent = MTP_PARENT_ROOT;
-                    entry.object_name = p.filename().string();
-                    entry.display_name = p.filename().string();
+                    entry.display_name = std::string(p.filename().string());
                     entry.path = p.string();
                     entry.object_format = MTP_FORMAT_ASSOCIATION;
                     entry.object_size = 0;
+                    entry.watch_fd = setup_dir_inotify(p);
 
                     db.insert( std::pair<MtpObjectHandle, DbEntry>(handle, entry) );
 
                     parse_directory (p, handle);
                 }
             } else
-                std::cout << p << " does not exist\n";
+                LOG(WARNING) << p << " does not exist\n";
         }
         catch (const filesystem_error& ex) {
-            std::cout << ex.what() << '\n';
+            LOG(ERROR) << ex.what();
         }
 
     }
 
+    void read_more_notify()
+    {
+        stream_desc.async_read_some(buf.prepare(buf.max_size()),
+                                    boost::bind(&UbuntuMtpDatabase::inotify_handler,
+                                                this,
+                                                asio::placeholders::error,
+                                                asio::placeholders::bytes_transferred));
+    }
+
+    void inotify_handler(const boost::system::error_code&,
+                        std::size_t transferred)
+    {
+        size_t processed = 0;
+     
+        while(transferred - processed >= sizeof(inotify_event))
+        {
+            const char* cdata = processed + asio::buffer_cast<const char*>(buf.data());
+            const inotify_event* ievent = reinterpret_cast<const inotify_event*>(cdata);
+            MtpObjectHandle parent;
+     
+            processed += sizeof(inotify_event) + ievent->len;
+     
+            BOOST_FOREACH(MtpObjectHandle i, db | boost::adaptors::map_keys) {
+                if (db.at(i).watch_fd == ievent->wd) {
+                    parent = i;
+                    break;
+                }
+            }
+
+            path p(db.at(parent).path + "/" + ievent->name);
+
+            if(ievent->len > 0 && ievent->mask & IN_MODIFY)
+            {
+                VLOG(2) << __PRETTY_FUNCTION__ << ": file modified: " << p.string();
+                BOOST_FOREACH(MtpObjectHandle i, db | boost::adaptors::map_keys) {
+                    if (db.at(i).path == p.string()) {
+                        try {
+                            VLOG(2) << "new size: " << file_size(p);
+                            db.at(i).object_size = file_size(p);
+                        } catch (const filesystem_error& ex) {
+                            PLOG(WARNING) << "There was an error reading file properties";
+                        }
+                    }
+                }
+            }
+            else if(ievent->len > 0 && ievent->mask & IN_CREATE)
+            {
+                VLOG(2) << __PRETTY_FUNCTION__ << ": file created: " << p.string();
+
+                /* try to deal with it as if it was a file. */
+                add_file_entry(p, parent);
+            }
+            else if(ievent->len > 0 && ievent->mask & IN_DELETE)
+            {
+                VLOG(2) << __PRETTY_FUNCTION__ << ": file deleted: " << p.string();
+                BOOST_FOREACH(MtpObjectHandle i, db | boost::adaptors::map_keys) {
+                    if (db.at(i).path == p.string()) {
+                        VLOG(2) << "deleting file at handle " << i;
+                        deleteFile(i);
+                        if (local_server)
+                            local_server->sendObjectRemoved(i);
+                        break;
+                    }
+                }
+            }
+        }
+     
+        read_more_notify();
+    }
+
 public:
-    UbuntuMtpDatabase(const char *dir) : counter(1)
+    UbuntuMtpDatabase(const char *dir):
+        counter(1),
+        stream_desc(io_svc),
+        buf(1024)
     {
 	std::string basedir (dir);
 
+        local_server = nullptr;
+
+        inotify_fd = inotify_init();
+        if (inotify_fd <= 0)
+            PLOG(FATAL) << "Unable to initialize inotify";
+
+        stream_desc.assign(inotify_fd);
+
 	db = std::map<MtpObjectHandle, DbEntry>();
 
+        notifier_thread = boost::thread(&UbuntuMtpDatabase::read_more_notify,
+                                       this);
+
 	readFiles(basedir + "/Documents");
 	readFiles(basedir + "/Music");
 	readFiles(basedir + "/Videos");
 	readFiles(basedir + "/Pictures");
 	readFiles(basedir + "/Downloads");
-	
-        std::cout << __PRETTY_FUNCTION__ << ": object count:" << db.size() << std::endl;
+
+        LOG(INFO) << "Added " << counter << " entries to the database.";
+
+        io_service_thread = boost::thread(boost::bind(&asio::io_service::run, &io_svc));
     }
 
-    virtual ~UbuntuMtpDatabase() {}
+    virtual ~UbuntuMtpDatabase() {
+        io_svc.stop();
+        notifier_thread.detach();
+        io_service_thread.join();
+        close(inotify_fd);
+    }
 
     // called from SendObjectInfo to reserve a database entry for the incoming file
     virtual MtpObjectHandle beginSendObject(
@@ -159,11 +324,10 @@ public:
         if (parent == 0)
             return kInvalidObjectHandle;
 
-        std::cout << __PRETTY_FUNCTION__ << ": " << path << " - " << parent << std::endl;
+        VLOG(1) << __PRETTY_FUNCTION__ << ": " << path << " - " << parent;
 
         entry.storage_id = storage;
         entry.parent = parent;
-        entry.object_name = std::string(basename(path.c_str()));
         entry.display_name = std::string(basename(path.c_str()));
         entry.path = path;
         entry.object_format = format;
@@ -185,11 +349,18 @@ public:
         MtpObjectFormat format,
         bool succeeded)
     {
-        std::cout << __PRETTY_FUNCTION__ << ": " << path << std::endl;
+        VLOG(1) << __PRETTY_FUNCTION__ << ": " << path;
 
 	if (!succeeded) {
-		db.erase(handle);
-	}
+            db.erase(handle);
+        } else {
+            boost::filesystem::path p (path);
+
+            if (format != MTP_FORMAT_ASSOCIATION) {
+                /* Resync file size, just in case this is actually an Edit. */
+                db.at(handle).object_size = file_size(p);
+            }
+        }
     }
 
     virtual MtpObjectHandleList* getObjectList(
@@ -197,7 +368,7 @@ public:
         MtpObjectFormat format,
         MtpObjectHandle parent)
     {
-        std::cout << __PRETTY_FUNCTION__ << ": " << storageID << ", " << format << ", " << parent << std::endl;
+        VLOG(1) << __PRETTY_FUNCTION__ << ": " << storageID << ", " << format << ", " << parent;
         MtpObjectHandleList* list = nullptr;
         try
         {
@@ -222,7 +393,7 @@ public:
         MtpObjectFormat format,
         MtpObjectHandle parent)
     {
-        std::cout << __PRETTY_FUNCTION__ << ": " << storageID << ", " << format << ", " << parent << std::endl;
+        VLOG(1) << __PRETTY_FUNCTION__ << ": " << storageID << ", " << format << ", " << parent;
         try
         {
             return db.size();
@@ -237,21 +408,42 @@ public:
     // results can be NULL
     virtual MtpObjectFormatList* getSupportedPlaybackFormats()
     {
-        std::cout << __PRETTY_FUNCTION__ << std::endl;
-        static const MtpObjectFormatList list = {MTP_FORMAT_PNG};
+        VLOG(1) << __PRETTY_FUNCTION__;
+        static const MtpObjectFormatList list = {
+            /* Generic files */
+            MTP_FORMAT_UNDEFINED,
+
+            /* Supported audio formats */
+            MTP_FORMAT_OGG,
+            MTP_FORMAT_MP3,
+            MTP_FORMAT_WAV,
+            MTP_FORMAT_WMA,
+            MTP_FORMAT_AAC,
+            MTP_FORMAT_FLAC,
+
+            /* Supported video formats */
+            // none listed yet, video apparently broken.
+
+            /* Audio album, and album art */
+            MTP_FORMAT_ABSTRACT_AUDIO_ALBUM,
+
+            /* Playlists for audio and video */
+            MTP_FORMAT_ABSTRACT_AV_PLAYLIST,
+        };
+
         return new MtpObjectFormatList{list};
     }
     
     virtual MtpObjectFormatList* getSupportedCaptureFormats()
     {
-        std::cout << __PRETTY_FUNCTION__ << std::endl;
+        VLOG(1) << __PRETTY_FUNCTION__;
         static const MtpObjectFormatList list = {MTP_FORMAT_ASSOCIATION, MTP_FORMAT_PNG};
         return new MtpObjectFormatList{list};
     }
     
     virtual MtpObjectPropertyList* getSupportedObjectProperties(MtpObjectFormat format)
     {
-        std::cout << __PRETTY_FUNCTION__ << std::endl;
+        VLOG(1) << __PRETTY_FUNCTION__;
 	/*
         if (format != MTP_FORMAT_PNG)
             return nullptr;
@@ -266,6 +458,7 @@ public:
             MTP_PROPERTY_WIDTH,
             MTP_PROPERTY_HEIGHT,
             MTP_PROPERTY_IMAGE_BIT_DEPTH,
+            MTP_PROPERTY_OBJECT_FILE_NAME,
             MTP_PROPERTY_DISPLAY_NAME            
         };
          
@@ -274,7 +467,7 @@ public:
     
     virtual MtpDevicePropertyList* getSupportedDeviceProperties()
     {
-        std::cout << __PRETTY_FUNCTION__ << std::endl;
+        VLOG(1) << __PRETTY_FUNCTION__;
         static const MtpDevicePropertyList list = { MTP_DEVICE_PROPERTY_UNDEFINED };
         return new MtpDevicePropertyList{list};
     }
@@ -284,7 +477,10 @@ public:
         MtpObjectProperty property,
         MtpDataPacket& packet)
     {        
-        std::cout << __PRETTY_FUNCTION__ << std::endl;
+        VLOG(1) << __PRETTY_FUNCTION__
+                << " handle: " << handle
+                << " property: " << MtpDebug::getObjectPropCodeName(property);
+
         switch(property)
         {
             case MTP_PROPERTY_STORAGE_ID: packet.putUInt32(db.at(handle).storage_id); break;            
@@ -292,6 +488,7 @@ public:
             case MTP_PROPERTY_OBJECT_FORMAT: packet.putUInt32(db.at(handle).object_format); break;
             case MTP_PROPERTY_OBJECT_SIZE: packet.putUInt32(db.at(handle).object_size); break;
             case MTP_PROPERTY_DISPLAY_NAME: packet.putString(db.at(handle).display_name.c_str()); break;
+            case MTP_PROPERTY_OBJECT_FILE_NAME: packet.putString(db.at(handle).display_name.c_str()); break;
             default: return MTP_RESPONSE_GENERAL_ERROR; break;                
         }
         
@@ -303,15 +500,56 @@ public:
         MtpObjectProperty property,
         MtpDataPacket& packet)
     {
-        std::cout << __PRETTY_FUNCTION__ << std::endl;
-        return MTP_RESPONSE_OPERATION_NOT_SUPPORTED;
+        DbEntry entry;
+        MtpStringBuffer buffer;
+        std::string oldname;
+        std::string newname;
+        path oldpath;
+        path newpath;
+
+        VLOG(1) << __PRETTY_FUNCTION__
+                << " handle: " << handle
+                << " property: " << MtpDebug::getObjectPropCodeName(property);
+
+        switch(property)
+        {
+            case MTP_PROPERTY_OBJECT_FILE_NAME:
+                try {
+                    entry = db.at(handle);
+
+                    packet.getString(buffer);
+                    newname = strdup(buffer);
+
+                    oldpath /= entry.path;
+                    newpath /= oldpath.branch_path() / "/" / newname;
+
+                    boost::filesystem::rename(oldpath, newpath);
+
+                    db.at(handle).display_name = newname;
+                    db.at(handle).path = newpath.string();
+                } catch (filesystem_error& fe) {
+                    LOG(ERROR) << fe.what();
+                    return MTP_RESPONSE_DEVICE_BUSY;
+                } catch (std::exception& e) {
+                    LOG(ERROR) << e.what();
+                    return MTP_RESPONSE_GENERAL_ERROR;
+                } catch (...) {
+                    LOG(ERROR) << "An unexpected error has occurred";
+                    return MTP_RESPONSE_GENERAL_ERROR;
+		}
+
+                break;
+            default: return MTP_RESPONSE_OPERATION_NOT_SUPPORTED; break;
+        }
+        
+        return MTP_RESPONSE_OK;
     }
 
     virtual MtpResponseCode getDevicePropertyValue(
         MtpDeviceProperty property,
         MtpDataPacket& packet)
     {
-        std::cout << __PRETTY_FUNCTION__ << std::endl;
+        VLOG(1) << __PRETTY_FUNCTION__;
         return MTP_RESPONSE_GENERAL_ERROR;
     }
 
@@ -319,14 +557,14 @@ public:
         MtpDeviceProperty property,
         MtpDataPacket& packet)
     {
-        std::cout << __PRETTY_FUNCTION__ << std::endl;
+        VLOG(1) << __PRETTY_FUNCTION__;
         return MTP_RESPONSE_OPERATION_NOT_SUPPORTED;
     }
 
     virtual MtpResponseCode resetDeviceProperty(
         MtpDeviceProperty property)
     {
-        std::cout << __PRETTY_FUNCTION__ << std::endl;
+        VLOG(1) << __PRETTY_FUNCTION__;
         return MTP_RESPONSE_OPERATION_NOT_SUPPORTED;
     }
 
@@ -338,7 +576,7 @@ public:
         int depth,
         MtpDataPacket& packet)
     {
-        std::cout << __PRETTY_FUNCTION__ << std::endl;
+        VLOG(2) << __PRETTY_FUNCTION__;
         return MTP_RESPONSE_OPERATION_NOT_SUPPORTED;
     }
 
@@ -346,28 +584,28 @@ public:
         MtpObjectHandle handle,
         MtpObjectInfo& info)
     {
-        std::cout << __PRETTY_FUNCTION__ << std::endl;
+        VLOG(2) << __PRETTY_FUNCTION__;
+
         info.mHandle = handle;
         info.mStorageID = db.at(handle).storage_id;
         info.mFormat = db.at(handle).object_format;
         info.mProtectionStatus = 0x0;
-        info.mCompressedSize = 0;
-        info.mThumbFormat = db.at(handle).object_format;
-        info.mThumbCompressedSize = 20*20*4;
-        info.mThumbPixWidth = 20;
-        info.mThumbPixHeight  =20;
-        info.mImagePixWidth = 20;
-        info.mImagePixHeight = 20;
-        info.mImagePixDepth = 4;
+        info.mCompressedSize = db.at(handle).object_size;
+        info.mImagePixWidth = 0;
+        info.mImagePixHeight = 0;
+        info.mImagePixDepth = 0;
         info.mParent = db.at(handle).parent;
         info.mAssociationType = 0;
         info.mAssociationDesc = 0;
         info.mSequenceNumber = 0;
-        info.mName = ::strdup(db.at(handle).object_name.c_str());
+        info.mName = ::strdup(db.at(handle).display_name.c_str());
         info.mDateCreated = 0;
         info.mDateModified = 0;
         info.mKeywords = ::strdup("ubuntu,touch");
         
+        if (VLOG_IS_ON(2))
+            info.print();
+
         return MTP_RESPONSE_OK;
     }
 
@@ -389,7 +627,7 @@ public:
     {
         DbEntry entry = db.at(handle);
 
-        std::cout << __PRETTY_FUNCTION__ << std::endl;
+        VLOG(1) << __PRETTY_FUNCTION__ << " handle: " << handle;
 
         outFilePath = std::string(entry.path);
         outFileLength = entry.object_size;
@@ -403,7 +641,10 @@ public:
         size_t orig_size = db.size();
         size_t new_size;
 
-        std::cout << __PRETTY_FUNCTION__ << std::endl;
+        VLOG(2) << __PRETTY_FUNCTION__ << " handle: " << handle;
+
+        if (db.at(handle).object_format == MTP_FORMAT_ASSOCIATION)
+            inotify_rm_watch(inotify_fd, db.at(handle).watch_fd);
 
         new_size = db.erase(handle);
 
@@ -416,25 +657,28 @@ public:
                 if (db.at(i).parent == handle)
                     db.erase(i);
             }
+
             return MTP_RESPONSE_OK;
         }
         else
             return MTP_RESPONSE_GENERAL_ERROR;
     }
 
-    /*
     virtual MtpResponseCode moveFile(MtpObjectHandle handle, MtpObjectHandle new_parent)
     {
-        std::cout << __PRETTY_FUNCTION__ << std::endl;
+        VLOG(1) << __PRETTY_FUNCTION__ << " handle: " << handle
+                << " new parent: " << new_parent;
 
         // change parent
+        db.at(handle).parent = new_parent;
 
-        return MTP_RESPONSE_OK
+        return MTP_RESPONSE_OK;
     }
 
+    /*
     virtual MtpResponseCode copyFile(MtpObjectHandle handle, MtpObjectHandle new_parent)
     {
-        std::cout << __PRETTY_FUNCTION__ << std::endl;
+        VLOG(2) << __PRETTY_FUNCTION__;
 
         // duplicate DbEntry
         // change parent
@@ -445,7 +689,7 @@ public:
 
     virtual MtpObjectHandleList* getObjectReferences(MtpObjectHandle handle)
     {
-        std::cout << __PRETTY_FUNCTION__ << std::endl;
+        VLOG(1) << __PRETTY_FUNCTION__;
         return nullptr;
     }
 
@@ -453,7 +697,7 @@ public:
         MtpObjectHandle handle,
         MtpObjectHandleList* references)
     {
-        std::cout << __PRETTY_FUNCTION__ << std::endl;
+        VLOG(1) << __PRETTY_FUNCTION__;
         return MTP_RESPONSE_OPERATION_NOT_SUPPORTED;    
     }
 
@@ -461,7 +705,8 @@ public:
         MtpObjectProperty property,
         MtpObjectFormat format)
     {
-        std::cout << __PRETTY_FUNCTION__ << std::endl;
+        VLOG(1) << __PRETTY_FUNCTION__ << MtpDebug::getObjectPropCodeName(property);
+
         MtpProperty* result = nullptr;
         switch(property)
         {
@@ -471,7 +716,8 @@ public:
             case MTP_PROPERTY_WIDTH: result = new MtpProperty(property, MTP_TYPE_UINT32); break;
             case MTP_PROPERTY_HEIGHT: result = new MtpProperty(property, MTP_TYPE_UINT32); break;
             case MTP_PROPERTY_IMAGE_BIT_DEPTH: result = new MtpProperty(property, MTP_TYPE_UINT32); break;
-            case MTP_PROPERTY_DISPLAY_NAME: result = new MtpProperty(property, MTP_TYPE_STR); break;
+            case MTP_PROPERTY_DISPLAY_NAME: result = new MtpProperty(property, MTP_TYPE_STR, true); break;
+            case MTP_PROPERTY_OBJECT_FILE_NAME: result = new MtpProperty(property, MTP_TYPE_STR, true); break;
             default: break;                
         }
         
@@ -480,18 +726,21 @@ public:
 
     virtual MtpProperty* getDevicePropertyDesc(MtpDeviceProperty property)
     {
-        std::cout << __PRETTY_FUNCTION__ << std::endl;
+        VLOG(1) << __PRETTY_FUNCTION__ << MtpDebug::getDevicePropCodeName(property);
         return new MtpProperty(MTP_DEVICE_PROPERTY_UNDEFINED, MTP_TYPE_UNDEFINED);
     }
     
-    virtual void sessionStarted()
+    virtual void sessionStarted(MtpServer* server)
     {
-        std::cout << __PRETTY_FUNCTION__ << std::endl;
+        VLOG(1) << __PRETTY_FUNCTION__;
+        local_server = server;
     }
 
     virtual void sessionEnded()
     {
-        std::cout << __PRETTY_FUNCTION__ << std::endl;
+        VLOG(1) << __PRETTY_FUNCTION__;
+        VLOG(1) << "objects in db at session end: " << db.size();
+        local_server = nullptr;
     }
 };
 }
